CREATE TABLE [privileges]
(
 [id] Int IDENTITY(1,1) NOT NULL,
 [name] nvarchar(45) NOT NULL UNIQUE,
 [created] Datetime2 DEFAULT CURRENT_TIMESTAMP NOT NULL
);
CREATE INDEX [privileges created] ON [privileges] ([created] DESC);
ALTER TABLE [privileges] ADD CONSTRAINT [PK_privileges] PRIMARY KEY ([id]);

CREATE TABLE [resources]
(
 [id] Int IDENTITY(1,1) NOT NULL,
 [name] nvarchar(45) NOT NULL UNIQUE,
 [created] Datetime2 DEFAULT CURRENT_TIMESTAMP NOT NULL
);
CREATE INDEX [resources created] ON [resources] ([created] DESC);
ALTER TABLE [resources] ADD CONSTRAINT [PK_resources] PRIMARY KEY ([id]);

CREATE TABLE [roles]
(
 [id] Int IDENTITY(1,1) NOT NULL,
 [name] nvarchar(45) NOT NULL UNIQUE,
 [parent_id] Int NULL,
 [created] Datetime2 DEFAULT CURRENT_TIMESTAMP NOT NULL
);
CREATE INDEX [parent role] ON [roles] ([parent_id]);
CREATE INDEX [roles created] ON [roles] ([created] DESC);
ALTER TABLE [roles] ADD CONSTRAINT [PK_roles] PRIMARY KEY ([id]);


CREATE TABLE [users_roles]
(
 [user_id] Int NOT NULL,
 [role_id] Int NOT NULL
);
CREATE INDEX [users roles user] ON [users_roles] ([user_id]);
CREATE INDEX [users roles role] ON [users_roles] ([role_id]);

CREATE TABLE [acl]
(
 [id] Int IDENTITY(1,1) NOT NULL,
 [role_id] Int NOT NULL,
 [privilege_id] Int NOT NULL,
 [resource_id] Int NOT NULL,
 [allowed] Smallint NOT NULL
);
CREATE INDEX [acl role] ON [acl] ([role_id]);
CREATE INDEX [acl privilege] ON [acl] ([privilege_id]);
CREATE INDEX [acl resource] ON [acl] ([resource_id]);
ALTER TABLE [acl] ADD CONSTRAINT [PK_acl] PRIMARY KEY ([id]);

CREATE TABLE [acl_log]
(
 [acl_id] Int NULL,
 [user_id] Int NOT NULL,
 [time] Datetime2 DEFAULT CURRENT_TIMESTAMP NOT NULL,
 [action] CHAR(6) NOT NULL CHECK ([action] IN ('insert', 'update', 'delete')),
 [role_name] nvarchar(45) NOT NULL,
 [resource_name] nvarchar(45) NOT NULL,
 [privilege_name] nvarchar(45) NOT NULL,
 [allowed] Smallint NOT NULL
);
CREATE INDEX [acl log time] ON [acl_log] ([time]);


ALTER TABLE [acl] ADD CONSTRAINT [acl privilege] FOREIGN KEY ([privilege_id]) REFERENCES [privileges] ([id]) ON UPDATE CASCADE ON DELETE CASCADE;
ALTER TABLE [acl] ADD CONSTRAINT [acl resource] FOREIGN KEY ([resource_id]) REFERENCES [resources] ([id]) ON UPDATE CASCADE ON DELETE CASCADE;
ALTER TABLE [acl] ADD CONSTRAINT [acl role] FOREIGN KEY ([role_id]) REFERENCES [roles] ([id]) ON UPDATE CASCADE ON DELETE CASCADE;
ALTER TABLE [acl_log] ADD CONSTRAINT [acl log] FOREIGN KEY ([acl_id]) REFERENCES [acl] ([id]) ON UPDATE CASCADE ON DELETE SET NULL;
ALTER TABLE [roles] ADD CONSTRAINT [parent role] FOREIGN KEY ([parent_id]) REFERENCES [roles] ([id]) ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE [users_roles] ADD CONSTRAINT [users roles role] FOREIGN KEY ([role_id]) REFERENCES [roles] ([id]) ON UPDATE CASCADE ON DELETE CASCADE;
ALTER TABLE [users_roles] ADD CONSTRAINT [users roles user] FOREIGN KEY ([user_id]) REFERENCES [users] ([id]) ON UPDATE CASCADE ON DELETE CASCADE;


--TODO: what should happen when user does not exist? should it try to insert NULL and throw an exception or do nothing
--if JOIN [users] u ON u.[user_name] = SUSER_SNAME() is modified with LEFT OUTER JOIN it tries to insert NULL
CREATE TRIGGER [acl_log_insert_trigger]
ON [acl]
AFTER INSERT
AS
BEGIN
	INSERT [acl_log](
                [acl_id],
                [user_id],
                [action],
                [role_name],
                [resource_name],
                [privilege_name],
                [allowed])
	SELECT ins.id, u.id, N'insert', r.name, res.name, p.name, ins.allowed
	FROM [inserted] ins
	JOIN [roles] r
		ON r.id = ins.role_id
	JOIN [resources] res
		ON res.id = ins.resource_id
	JOIN [privileges] p
		ON p.id = ins.privilege_id
	JOIN [users] u
		ON u.[user_name] = SUSER_SNAME()
END;

CREATE TRIGGER [acl_log_update_trigger]
ON [acl]
AFTER UPDATE
AS
BEGIN
	INSERT [acl_log](
                [acl_id],
                [user_id],
                [action],
                [role_name],
                [resource_name],
                [privilege_name],
                [allowed])
	SELECT upd.id, u.id, N'update', r.name, res.name, p.name, upd.allowed
	FROM [inserted] upd
	JOIN [roles] r
		ON r.id = upd.role_id
	JOIN [resources] res
		ON res.id = upd.resource_id
	JOIN [privileges] p
		ON p.id = upd.privilege_id
	JOIN [users] u
		ON u.[user_name] = SUSER_SNAME()
END;

CREATE TRIGGER [acl_log_delete_trigger]
ON [acl]
AFTER DELETE
AS
BEGIN
	INSERT [acl_log](
                [user_id],
                [action],
                [role_name],
                [resource_name],
                [privilege_name],
                [allowed])
	SELECT u.id, N'delete', r.name, res.name, p.name, del.allowed
	FROM [deleted] del
	JOIN [roles] r
		ON r.id = del.role_id
	JOIN [resources] res
		ON res.id = del.resource_id
	JOIN [privileges] p
		ON p.id = del.privilege_id
	JOIN [users] u
		ON u.[user_name] = SUSER_SNAME()
END;