CREATE TABLE `privileges`
(
  `id` Int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `name` Varchar(45) CHARACTER SET utf8 COLLATE utf8_general_ci UNIQUE NOT NULL,
  `created` Timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
 PRIMARY KEY (id)
) ENGINE = InnoDB
  DEFAULT CHARACTER SET utf8
  COLLATE utf8_general_ci
  ROW_FORMAT = Compact
;

CREATE INDEX `privileges created` USING BTREE ON `privileges` (`created` DESC);

-- Table resources

CREATE TABLE `resources`
(
  `id` Int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `name` Varchar(45) CHARACTER SET utf8 COLLATE utf8_general_ci UNIQUE NOT NULL,
  `created` Timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
 PRIMARY KEY (id)
) ENGINE = InnoDB
  DEFAULT CHARACTER SET utf8
  COLLATE utf8_general_ci
  ROW_FORMAT = Compact
;

CREATE INDEX `resources created` USING BTREE ON `resources` (`created` DESC);

-- Table roles

CREATE TABLE `roles`
(
  `id` Int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `name` Varchar(45) CHARACTER SET utf8 COLLATE utf8_general_ci UNIQUE NOT NULL,
  `parent_id` Int(10) UNSIGNED,
  `created` Timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
 PRIMARY KEY (id)
) ENGINE = InnoDB
  DEFAULT CHARACTER SET utf8
  COLLATE utf8_general_ci
  ROW_FORMAT = Compact
;

CREATE INDEX `parent role` USING BTREE ON `roles` (`parent_id`);
CREATE INDEX `roles created` USING BTREE ON `roles` (`created` DESC);


-- Table users_roles

CREATE TABLE users_roles
(
  user_id Int(10) UNSIGNED NOT NULL,
  role_id Int(10) UNSIGNED NOT NULL
) ENGINE = InnoDB
  DEFAULT CHARACTER SET utf8
  COLLATE utf8_general_ci
  ROW_FORMAT = Compact
;

CREATE INDEX `users roles user` USING BTREE ON users_roles (user_id);
CREATE INDEX `users roles role` USING BTREE ON users_roles (role_id);

CREATE TABLE `acl`
(
  `id` Int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
  `role_id` Int(10) UNSIGNED NOT NULL,
  `privilege_id` Int(10) UNSIGNED NOT NULL,
  `resource_id` Int(10) UNSIGNED NOT NULL,
  `allowed` Tinyint(1) NOT NULL,
 PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARACTER SET utf8
  COLLATE utf8_general_ci
  ROW_FORMAT = Compact
;

CREATE INDEX `acl role` USING BTREE ON `acl` (`role_id`);
CREATE INDEX `acl privilege` USING BTREE ON `acl` (`privilege_id`);
CREATE INDEX `acl resource` USING BTREE ON `acl` (`resource_id`);

-- Table acl_log

CREATE TABLE `acl_log`
(
  `acl_id` Int(10) UNSIGNED,
  `user_id` Int(10) UNSIGNED NOT NULL,
  `time` Timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `action` Enum('insert','update','delete') CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
  `role_name` Varchar(45) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
  `resource_name` Varchar(45) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
  `privilege_name` Varchar(45) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
  `allowed` Tinyint(1) NOT NULL
) ENGINE = InnoDB
  DEFAULT CHARACTER SET utf8
  COLLATE utf8_general_ci
  ROW_FORMAT = Compact
;

CREATE INDEX `acl log time` USING BTREE ON `acl_log` (`time`);



-- constraints

ALTER TABLE `acl` ADD CONSTRAINT `acl privilege` FOREIGN KEY (privilege_id) REFERENCES privileges (id) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE `acl` ADD CONSTRAINT `acl resource` FOREIGN KEY (resource_id) REFERENCES resources (id) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE `acl` ADD CONSTRAINT `acl role` FOREIGN KEY (role_id) REFERENCES roles (id) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE `acl_log` ADD CONSTRAINT `acl log` FOREIGN KEY (acl_id) REFERENCES acl (id) ON DELETE SET NULL ON UPDATE CASCADE;

ALTER TABLE `roles` ADD CONSTRAINT `parent role` FOREIGN KEY (parent_id) REFERENCES roles (id) ON DELETE RESTRICT ON UPDATE RESTRICT;

ALTER TABLE `users_roles` ADD CONSTRAINT `users roles role` FOREIGN KEY (role_id) REFERENCES roles (id) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE `users_roles` ADD CONSTRAINT `users roles user` FOREIGN KEY (user_id) REFERENCES users (id) ON DELETE CASCADE ON UPDATE CASCADE;